@@ -100,13 +100,6 @@ def to_graphviz(graph, include_src=None):
       if next_node in block.exits_from_end:
         edge_attrs = {}
         edge_attrs['style'] = 'solid'
-        branches = block.branches | block.except_branches | block.reraise_branches
-        if branches.get(True) == next_node:
-          edge_attrs['color'] = 'red'
-        if branches.get(False) == next_node:
-          edge_attrs['color'] = 'blue'
-        if branches.get(True) == next_node and block.branches.get(False) == next_node:
-          edge_attrs['color'] = 'purple'
         g.add_edge(node_id, next_node_id, **edge_attrs)
 
   if include_src is not None:
